#!/bin/sh

DOC_DIR=atmel_docs
LNK_DIR=links
PDFLNK_DIR=pdflinks
CLK_DIR=clocks
HDR_DIR=headers
XML_DIR=xml

mkdir -p ${DOC_DIR}
mkdir -p ${LNK_DIR}
mkdir -p ${CLK_DIR}
mkdir -p ${PDFLNK_DIR}
mkdir -p ${HDR_DIR}
mkdir -p ${XML_DIR}

./get_docs.sh ${DOC_DIR}
./convert_to_txt.sh ${DOC_DIR}

rm {LNK_DIR}/* ${PDFLNK_DIR}/*
./make_links.sh ${DOC_DIR} ${LNK_DIR} ${PDFLNK_DIR}
./parse_doc.py ${LNK_DIR} ${CLK_DIR}

echo "-- generating header files --"

cd ${LNK_DIR}
rm ../${HDR_DIR}/*
for i in * ; do
    if [ -f ../${XML_DIR}/$i.xml ]; then
	echo $i
	../gen_regs.py ../${XML_DIR}/$i.xml ../${CLK_DIR}/$i > ../${HDR_DIR}/aversive/$i.h
    fi
done
cd -

cd ${HDR_DIR}
EL=
cat <<EOF > aversive/parts.h
/*  
 *  Copyright Droids Corporation, Microb Technology, Eirbot (2007)
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id $
 *
 */

/* WARNING : this file is automatically generated by scripts.
 * You should not edit it. If you find something wrong in it,
 * write to zer0@droids-corp.org */

#ifndef _AVERSIVE_PARTS_H_
#define _AVERSIVE_PARTS_H_

EOF
for i in * ; do
    part=${i%.h}
    part=${part#aversive_}
    if [ "$part" = "parts" ]; then
	continue
    fi
    echo "#${EL}if defined (__AVR_${part}__)" >> aversive/parts.h
    echo "#include <$i>" >> aversive/parts.h
    EL=el
done
echo "#else" >> aversive/parts.h
echo "#error \"This arch is not implemented yet\"" >> aversive/parts.h
echo "#endif" >> aversive/parts.h
echo >> aversive/parts.h
echo "#endif /* _AVERSIVE_PARTS_H_ */" >> aversive/parts.h
cd -

